package org.qatools.validator;


import java.util.List;

import org.apache.log4j.Logger;


/**
 * <pre>
 * Scott Wilson presents QATools.org
 * Specializing in Functional & Integration Test Automation using Maven Java Junit TestNG
 * 
 * 
 * Validator extends Junit by capturing and reporting all failures at once.
 * This sppeds your QA effort by reducing the number of iterations.
 * 
 * Junit pukes at the first failed assertion and also does not enforce useful error messages
 * 			assertTrue(false);		<=== FAILS IMMEDIATELY
 * 			assertEquals(1, 2)		
 * 			assertNotNull(null);	
 * 
 * Validator example - the API forces you to enter useful error messages
 * 			Validator v = new Validator();
 * 			v.assertTrue("Foo msg", false);
 * 			v.assertEquals("Bar msg", 1, 2);
 * 			v.assertNotNull("Apple", null);
 * 			v.throwExceptionIfErrors();		<=== Captures ALL errors not just the first one!
 * 
 * </pre>
 * 
 * @author Scott Wilson - QATools.org
 */
public class BasicValidator extends AValidator {									
    private static Logger logger = Logger.getLogger(BasicValidator.class);

    // ------------------------------------------------------
    // ----- constructors
    // ------------------------------------------------------

	public BasicValidator() {
        //do nothing constructor
    }
    
    
    // ------------------------------------------------------
    // ----- Re-implements these Junit APIs
    // ------------------------------------------------------

    
	/**
	 * Asserts that a condition is true. If it isn't it adds the error to the error list..
	 */
	public void assertTrue(String message, boolean condition) {
        if (!condition) {
			addError(message + " expected True but was False");
		}
	}
	

	
	/**
	 * Asserts that a condition is false. If it isn't it adds the error to the error list.
	 */
	public void assertFalse(String message, boolean condition) {
 		if (condition) {
			addError(message + " expected False but was True");
 		}
 	}
	


	/**
	 * Adds the message to the error list
	 */
	public void fail(String message) {
		addError(message);
	}



	/**
	 * Asserts that two objects are equal. If they are not
	 * add the message to the error list
	 */
	public void assertEquals(String message, Object expected, Object actual) {
		if (expected == null && actual == null) {
	    	return;
		}
		if (expected != null && expected.equals(actual)) {
			return;
		}
		addError(formatErrorMsg(message, expected, actual));
	}


	/**
	 * Asserts that two Strings are equal. 
	 */
	public void assertEquals(String message, String expected, String actual) {
		if (cleanString(expected) == null && cleanString(actual) == null) {
			return;
		}
		if (cleanString(expected) != null && cleanString(expected).equals(cleanString(actual))) {
			return;
		}
		addError(formatErrorMsg(message, cleanStringForReport(expected), cleanStringForReport(actual)));
	}


	
	/**
	 * Asserts that two doubles are equal concerning a delta.  If the expected
	 * value is infinity then the delta value is ignored.
	 */
	public void assertEquals(String message, double expected, double actual, double delta) {
		// handle infinity specially since subtracting to infinite values gives NaN and the
		// the following test fails
		if (Double.isInfinite(expected)) {
			if (!(expected == actual)) {
				addError(formatErrorMsg(message, new Double(expected), new Double(actual)));
			}
		} else if (!(Math.abs(expected-actual) <= delta)) { // Because comparison with NaN always returns false
			addError(formatErrorMsg(message, new Double(expected), new Double(actual)));
		}
	}
	


	/**
	 * Asserts that two floats are equal concerning a delta. If they are not
	 * an AssertionFailedError is thrown with the given message.  If the expected
	 * value is infinity then the delta value is ignored.
	 */
	public void assertEquals(String message, float expected, float actual, float delta) {
 		// handle infinity specially since subtracting to infinite values gives NaN and the
		// the following test fails
		if (Float.isInfinite(expected)) {
			if (!(expected == actual)) {
				addError(formatErrorMsg(message, new Float(expected), new Float(actual)));
			}
		} else if (!(Math.abs(expected-actual) <= delta)) {
			addError(formatErrorMsg(message, new Float(expected), new Float(actual)));			
		}
	}

	
	/**
	 * Asserts that two longs are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
	public void assertEquals(String message, long expected, long actual) {
		if (expected != actual) {
	    	addError(formatErrorMsg(message, new Long(expected), new Long(actual)));
		}
	}
	
	/**
	 * Asserts that two booleans are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
	public void assertEquals(String message, boolean expected, boolean actual) {
		if (expected != actual) {
	    	addError(formatErrorMsg(message, new Boolean(expected), new Boolean(actual)));
		}
  	}
	
	/**
	 * Asserts that two bytes are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
  	public void assertEquals(String message, byte expected, byte actual) {
		if (expected != actual) {
	    	addError(formatErrorMsg(message, new Byte(expected), new Byte(actual)));
		}
	}

  	
	/**
	 * Asserts that two chars are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
  	public void assertEquals(String message, char expected, char actual) {
		if (expected != actual) {
			addError(formatErrorMsg(message, new Character(expected), new Character(actual)));
		}
  	}

	/**
	 * Asserts that two shorts are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
	public void assertEquals(String message, short expected, short actual) {
		if (expected != actual) {
			addError(formatErrorMsg(message, new Short(expected), new Short(actual)));
		}
	}

	/**
	 * Asserts that two ints are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
  	public void assertEquals(String message, int expected, int actual) {
		if (expected != actual) {
			addError(formatErrorMsg(message, new Integer(expected), new Integer(actual)));
		}
  	}

	/**
	 * Asserts that an object isn't null. 
	 */
	public void assertNotNull(String message, Object object) {
		if (object == null) {
			addError(message + " expected not null, but was null");
		}
	}
	
	/**
	 * Asserts that an object is null. 
	 */
	public void assertNull(String message, Object object) {
		if (object != null) {
			addError(message + " expected null, but was not null");
		}
	}

	   /**
     * Asserts that an object is null. 
     */
    public void assertNull(String message, String object) {
        if (cleanString(object) != null) {
            addError(message + " expected null, but was not null - " + cleanStringForReport(object));
        }
    }

	
    // ------------------------------------------------------
    // ----- 
    // ------------------------------------------------------

	/**
	 * asserts list is not null and not empty
	 * @param list
	 */
	public <T extends Object> void assertListNotNullNotEmpty(String message, List<T> list) {
        if (list == null) {
			addError(message + " expected notNullNotEmpty, but was null");
        } else if (list.size() == 0) {
			addError(message + " expected notNullNotEmpty, but was empty");
        }        
	}
	
	/**
	 * asserts list is not null and not empty
	 * @param list
	 */
	public void assertStrNotNullNotEmpty(String message, String str) {
        if (cleanString(str) == null) {
			addError(message + " expected notNullNotEmpty, but was null - " + cleanStringForReport(str));
        } else if (cleanString(str).length() == 0) {
			addError(message + " expected notNullNotEmpty, but was empty - " + cleanStringForReport(str));
        }        
	}
	
    // ------------------------------------------------------
    // ----- Greater Than
    // ------------------------------------------------------

	/**
	 * actual should be greater than expected
	 * 
	 * @param message
	 * @param smallExpected
	 * @param bigActual
	 */
	public void assertGreaterThan(String message, long smallExpected, long bigActual) {
       if (!(smallExpected < bigActual)) {
			addError(message + " " + smallExpected + " NOT < " + bigActual);
       }
	}
	/**
	 * actual should be greater than expected
	 * 
	 * @param message
	 * @param smallExpected
	 * @param bigActual
	 */
	public void assertGreaterThan(String message, int smallExpected, int bigActual) {
       if (!(smallExpected < bigActual)) {
			addError(message + " " + smallExpected + " NOT < " + bigActual);
       }
	}

	
	/**
	 * actual should be greater than expected
	 * 
	 * @param message
	 * @param smallExpected
	 * @param bigActual
	 */
	public void assertGreaterThanEquals(String message, long smallExpected, long bigActual) {
       if (!(smallExpected <= bigActual)) {
			addError(message + " " + smallExpected + " NOT <= " + bigActual);
       }
	}
	/**
	 * actual should be greater than expected
	 * 
	 * @param message
	 * @param smallExpected
	 * @param bigActual
	 */
	public void assertGreaterThanEquals(String message, int smallExpected, int bigActual) {
       if (!(smallExpected <= bigActual)) {
			addError(message + " " + smallExpected + " NOT <= " + bigActual);
       }
	}

    // ------------------------------------------------------
    // ----- Less Than
    // ------------------------------------------------------

    /**
     * actual should be greater than expected
     * 
     * @param message
     * @param bigExpected
     * @param smallActual
     */
    public void assertLessThan(String message, long bigExpected, long smallActual) {
       if (!(bigExpected > smallActual)) {
            addError(message + " " + bigExpected + " NOT > " + smallActual);
       }
    }
    /**
     * actual should be greater than expected
     * 
     * @param message
     * @param bigExpected
     * @param smallActual
     */
    public void assertLessThan(String message, int bigExpected, int smallActual) {
       if (!(bigExpected > smallActual)) {
            addError(message + " " + bigExpected + " NOT > " + smallActual);
       }
    }

    
    /**
     * actual should be greater than expected
     * 
     * @param message
     * @param bigExpected
     * @param smallActual
     */
    public void assertLessThanEquals(String message, long bigExpected, long smallActual) {
       if (!(bigExpected >= smallActual)) {
            addError(message + " " + bigExpected + " NOT >= " + smallActual);
       }
    }
    /**
     * actual should be greater than expected
     * 
     * @param message
     * @param bigExpected
     * @param smallActual
     */
    public void assertLessThanEquals(String message, int bigExpected, int smallActual) {
       if (!(bigExpected >= smallActual)) {
            addError(message + " " + bigExpected + " NOT > " + smallActual);
       }
    }
    
    // ------------------------------------------------------
    // ----- 
    // ------------------------------------------------------

	/**
	 * The actual list should contain the expected object
	 * 
	 * @param expected
	 * @param actual
	 */
	public <T extends Object> void assertContains(String message, T expected, List<T> actual) {
        if (actual==null) {
			addError(message + " actual is NULL");
        	
        } else if (actual.size() == 0) {
			addError(message + " actual size is zero");
        	
        } else if (actual.contains(expected)) {
        	// do nothing
        } else {
			addError(message + " actual '" + actual.toString() + "' does NOT contain expected '" + expected + "'");
        }
	}
	
	/**
	 * The actual list should contain the expected object
	 * 
	 * @param expected
	 * @param actual
	 */
	public <T extends Object> void assertContains(String message, List<T> expected, T actual) {
        if (expected==null) {
			addError(message + " expected is NULL");
        	
        } else if (expected.size() == 0) {
			addError(message + " expected size is zero");

        } else if (expected.contains(actual)) {
        	// do nothing
        } else {
			addError(message + " expected '" + expected.toString() + "' does NOT contain actual '" + actual + "'");
        }
	}

	/**
	 * The actual string must be contained within the expected
	 * 
	 * @param message
	 * @param longExpected
	 * @param shortActual
	 */
	public void assertStrContainsStr(String message, String longExpected, String shortActual) {
        String tmpLongExpected = cleanString(longExpected);
        String tmpShortActual = cleanString(shortActual);
        
        if (tmpLongExpected==null && tmpShortActual==null) {
        	// do nothing
        	
        } else if (tmpLongExpected.contains(tmpShortActual)) {
        	// do nothing

        } else {
			addError(message + " expected '" + longExpected + "' does NOT contain actual '" + shortActual + "'");
        }
	}

	/**
	 * The expected must be contained within the actual
	 * 
	 * @param message
	 * @param shortExpected
	 * @param longActual
	 */
	public void assertStrContainsStr2(String message, String shortExpected, String longActual) {
        String tmpShortExpected = cleanString(shortExpected);
        String tmpLongActual = cleanString(longActual);
         
        if (tmpLongActual==null && tmpShortExpected==null) {
        	// do nothing
        	
        } else if (tmpLongActual.contains(tmpShortExpected)) {
        	// do nothing

        } else {
			addError(message + " actual '" + cleanStringForReport(longActual) + "' does NOT contain expected '" + cleanStringForReport(shortExpected) + "'");
        }
	}

	// ------------------------------------------------------
    // ----- 
    // ------------------------------------------------------

	/**
	 * 
	 * @param message
	 * @param list
	 * @param list2
	 */
	public void assertEquals(String message, List<Object> expected, List<Object> actual) {
        if (expected==null && actual!=null) {
        	String msg = message + " expected is null, actual is not null(size:" + actual.size() + ")";
			addError(msg);
        	return;
        	
        } else if (expected!=null && actual==null) {
        	String msg = message + " expected is not null(size:" + expected.size() + ") actual is null)";
			addError(msg);
        	return;        	
        } 

        if (expected.size() != actual.size()) {
        	String msg = message + " expected.size=" + expected.size() + ", actual.size=" + actual.size();
			addError(msg);
	    	addDebug(message + ", expected=" + expected.toString());
	    	addDebug(message + ", actual=" + actual.toString());
        	return;        	        	
        }
        
        boolean flag = true;
        for (int i=0; i<expected.size(); i++) {
        	Object e = expected.get(i);
        	Object a = actual.get(i);
        	if (!e.equals(a)) {
        		flag = false;
        	}
        }
        if (flag) {
        	// do nothing

        } else {
        	String msg = message + ", expected.list does NOT match actual.list";
			addError(msg);
	    	addDebug(message + ", expected=" + expected.toString());
	    	addDebug(message + ", actual=" + actual.toString());        	
        }
        
	}
	
	// ------------------------------------------------------
    // ----- assertNotEquals
    // ------------------------------------------------------

	/**
	 * Asserts that two objects are NOT equal. If they are equal
	 * add the message to the error list
	 */
	public void assertNotEquals(String message, Object expected, Object actual) {
        String msg = message + " - NOT expected:'" + expected + "', actual:'" + actual + "'";
		if (expected == null && actual == null) {
			addError(msg);
	    	return;
		}
		if (expected != null && expected.equals(actual)) {
			addError(msg);
			return;
		}
	}


	/**
	 * Asserts that two Strings are NOT equal. 
	 */
	public void assertNotEquals(String message, String expected, String actual) {
        String ex = cleanString(expected);
        String a = cleanString(actual);
        String msg = message + " - NOT expected:'" + ex + "', actual:'" + a + "'";

		if (ex == null && a == null) {
			addError(msg);
			return;
		}
		if (ex != null && ex.equals(a)) {
			addError(msg);
			return;
		}
	}

	
	/**
	 * Asserts that two longs are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
	public void assertNotEquals(String message, long expected, long actual) {
        String msg = message + " - NOT expected:'" + expected + "', actual:'" + actual + "'";
        
		if (expected == actual) {
    		addError(msg);
		}
	}
	
	/**
	 * Asserts that two booleans are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
	public void assertNotEquals(String message, boolean expected, boolean actual) {
        String msg = message + " - NOT expected:'" + expected + "', actual:'" + actual + "'";

		if (expected == actual) {
	    	addError(msg);
		}
  	}
	

	/**
	 * Asserts that two ints are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
  	public void assertNotEquals(String message, int expected, int actual) {
        String msg = message + " - NOT expected:'" + expected + "', actual:'" + actual + "'";

		if (expected == actual) {
			addError(msg);
		}
  	}
	
	// ------------------------------------------------------
    // ----- range
    // ------------------------------------------------------

  	/**
  	 * 
  	 * @param message
  	 * @param expectedLow
  	 * @param expectedHigh
  	 * @param actual
  	 */
	public void assertRange(String message, int expectedLow, int expectedHigh, int actual) {
        String msg = message + " - expected range:'" + expectedLow + "-" + expectedHigh + "', actual:'" + actual + "'";

		if ((expectedLow <= actual) && (actual <= expectedHigh)) {
			// do nothing
		} else {
			addError(msg);
		}
	}

  	/**
  	 * 
  	 * @param message
  	 * @param expectedLow
  	 * @param expectedHigh
  	 * @param actual
  	 */
	public void assertRange(String message, long expectedLow, long expectedHigh, long actual) {
        String msg = message + " - expected range:'" + expectedLow + "-" + expectedHigh + "', actual:'" + actual + "'";

		if ((expectedLow <= actual) && (actual <= expectedHigh)) {
			// do nothing
		} else {
			addError(msg);
		}
	}


	
}
